/*
	Sokonux - 1.0.3
  Copyright (C) 2007  Daniel Ponsoda Montiel

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "Sistema.h"
#include "Widget.h"

void* Widget::foco=NULL;

/**
 * Constructor de la clase Widget indicando tamao
 * @param cuadroPadre Indica cual es su cuadro padre (en general a que ventana
 *                    pertenece). Si esta a 0, el padre es la pantalla
 * @param x           Posicion horizontal relativa al padre
 * @param y           Posicion vertical relativa al padre
 * @param an          Anchura del widget
 * @param al          Altura del widget
 */
Widget::Widget(CUADRO** cuadroPadre, int x, int y, int ancho, int alto)
{
  // Obtener la posicion y tamao de este cuadro
  cuadro = new CUADRO;
  if (ancho==0) ancho=ANCHO;
  if (alto==0) alto=ALTO;
  cuadro->x  = x;
  cuadro->y  = y;
  cuadro->an = ancho;
  cuadro->al = alto;
  flags=0;

  // Obtener el cuadro padre
  if (cuadroPadre)
    this->cuadro->padre = *cuadroPadre;
  else{
    this->cuadro->padre = 0;
  }

  graficos=NULL;
}

/**
 * Constructor de la clase Widget a partir de un fichero de imagen
 * @param cuadroPadre Indica cual es su cuadro padre (en general a que ventana
 *                    pertenece). Si esta a 0, el padre es la pantalla
 * @param x           Posicion horizontal relativa al padre
 * @param y           Posicion vertical relativa al padre
 * @param ficheroImagen nombre del fichero de la imagen a establecer
 */
Widget::Widget(CUADRO** cuadroPadre, int x, int y, char* ficheroImagen)
{
  // Obtener la posicion y tamao de este cuadro
  this->cuadro = new CUADRO;
  this->cuadro->x  = x;
  this->cuadro->y  = y;

  // Obtener el cuadro padre
  if (cuadroPadre)
    this->cuadro->padre = *cuadroPadre;
  else{
    this->cuadro->padre = 0;
  }

  flags=0;

  // Obtener los graficos si se ha indicado
  if (ficheroImagen){
    graficos = Sistema::cargarImagen(ficheroImagen, &this->cuadro->an,
                                 &this->cuadro->al, COLOR_CLAVE);
  }
  else{
    graficos=NULL;
    this->cuadro->an=ANCHO;
    this->cuadro->al=ALTO;
  }

}

/**
 * Destructor de la clase Widget.
 * Libera los graficos del widget si fueron cargados.
 */
Widget::~Widget()
{

  // Liberar la superficie grafica de la imagen
  if (graficos) {
    Sistema::liberarImagen(graficos);
    graficos=NULL;
  }
}

/**
 * Dibuja en su posicon actual el mapa de bits del widget o su cuadro.
 */
void Widget::dibujar()
{
  int x,y;

  // Si no es visible, no dibujar
  if (flags & WID_INVISIBLE) return;

  // Obtener la posicion absoluta del widget en la pantalla
  x = xAbs(cuadro);
  y = yAbs(cuadro);

  CUADRO c = { x, y, cuadro->an, cuadro->al };

  // Si tiene mapa de bits, dibujar la imagen en la posicion establecida
  if (graficos) Sistema::dibujarImagen(graficos, x, y);

  // Si no, dibujar el cuadro
  else Sistema::dibujarCuadro(c, colorFondo, colorBorde);
}


/**
 * Detecta el estado del raton sobre el widget
 * @return MSG_DETECTADO_RATON si se ha detectado el raton.
 * @return MSG_RATON_CLICK si el raton esta pulsado sobre el widget
 */
DWORD Widget::detectarRaton()
{
  int    x = xAbs(cuadro);
  int    y = yAbs(cuadro);
  CUADRO c = { x, y, cuadro->an, cuadro->al };

  DWORD msg;

  // Si hay una imagen, detectar ignorando el color transparente
  if (graficos)   msg = Sistema::detectarRaton(x,y,graficos);

  // Si no comprobar todo el cuadro
  else            msg = Sistema::detectarRaton(c);

  // Y devolver el resultado
  return msg;
}


/**
 * Copia el puntero del grafico de otro widget a este widget
 * @param imagen Puntero al widget que se establecera a este
 */
void Widget::establecerImagen(Widget* imagen)
{
  graficos = imagen->graficos;

  cuadro->an = imagen->cuadro->an;
  cuadro->al = imagen->cuadro->al;
}

/**
 * Centra este widget sobre su widget padre segun los
 * valores dados en el atributo flags
 */
void Widget::centrar(int flags)
{
  int ancho, alto;

  // Comprobar si hay un widget padre
  if (cuadro->padre){
    ancho = cuadro->padre->an;
    alto  = cuadro->padre->al;
  }

  // Si no, se centrara sobre la pantalla
  else{
    ancho=ANCHO; alto=ALTO;
  }

  // Centrar las X
  if (flags&WID_CENTRARX)
    cuadro->x = (ancho - cuadro->an) / 2;

  // Centrar las Y
  if (flags&WID_CENTRARY)
    cuadro->y = (alto  - cuadro->al) / 2;

}

/**
 * Calcula recursivamente la posicion horizontal
 * absoluta del widget en la pantalla
 * @param cuadro Cuadro hijo que se va a calcular
 * @return Devuelve la posicion x del widget
 */
int Widget::xAbs(CUADRO* cuadro)
{
  if (!cuadro) return 0;
  if (cuadro->padre){
    return cuadro->x+xAbs(cuadro->padre);
  }
  else
    return cuadro->x;
}

/**
 * Calcula recursivamente la posicion vertical
 * absoluta del widget en la pantalla
 * @param cuadro Cuadro hijo que se va a calcular
 * @return Devuelve la posicion y del widget
 */
int Widget::yAbs(CUADRO* cuadro)
{
  if (!cuadro) return 0;
  if (cuadro->padre){
    return cuadro->y+yAbs(cuadro->padre);
  }
  else
    return cuadro->y;
}
